/*


LED game with 8 LED powered light domes with ohmic contact sense

"Light touch" (tm)


*/

#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "main.h"
#include "globals.h"
#include "misc.h"
#include "game.h"
#include "test.h"

int main(void) {
    // IO init

    DDRB  = (1<<PB7) | (1<<PB6) | (1<<PB2);
    PORTB = ~((1<<PB2) | (1<<PB1) | (1<<PB0)) ;  // turn on pull ups for unsused inputs
    DDRC  = 0;          // all inputs most of the time, external pull ups
    PORTC = 0;
    DDRD  = 0xFF;       // all outputs drive 1x WS2812B LEDs
    PORTD = 0;
     
    // ADC nit
    // VREF=VCC, left adjust result (8 bit mode)
    // ADC enable, prescaler 64 (sampling rate ~9600 smps)        
    ADMUX  = (1<<REFS0) | (1<<ADLAR);
    ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1); 
    read_ADC(0);    // dummy read, ADC startup

    // init timer 0
    // mode 2, CTC, top = OCR0A, prescaler 1024
    // F_ISR ~100 Hz
    TCCR0A = (1<<WGM01);
    TCCR0B = (1<<CS02) | (1<<CS00);
    OCR0A = 77;
    TIMSK0 |= (1<<OCIE0A); 

    sei();

    calibrate_matrix(matrix_cal);

    // endless main loop
    
    while(1) {
        set_sleep_mode(SLEEP_MODE_IDLE);
        sleep_mode();
        // continuously generate a random number
        // to avoid repeatable sequences for each power up cycle
        rand();             
        
        if (enable_light_show) {
            light_show();
        }

        if (flag_100ms) {
            flag_100ms=0;
            scan_inputs(matrix, 0);
            power_bank_keep_alive();  
            game();
        }
    }
}

// F_ISR ~100 Hz

ISR(TIMER0_COMPA_vect) {
    static uint8_t cnt100ms;

    cnt100ms++;
    if (cnt100ms == 10) {
        cnt100ms=0;
        flag_100ms=1;
    }
}
